@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,6 +21,8 @@
 
 # The Character object (and friends).
 
+from __future__ import print_function
+
 import renpy.display
 
 import re
@@ -138,26 +140,33 @@ def predict_show_display_say(who, what, who_args, what_args, window_args, image=
         return
 
 
-def compute_widget_properties(who_args, what_args, window_args, properties, variant=None):
+def compute_widget_properties(who_args, what_args, window_args, properties, variant=None, multiple=None):
     """
     Computes and returns the widget properties.
     """
 
-    def style_args(d):
+    def style_args(d, name):
 
-        if "style" not in d:
-            return d
+        style = d.get("style", None)
+
+        if style is None:
+            if multiple is None:
+                return d
+            else:
+                style = name
 
         in_rollback = renpy.exports.in_rollback()
 
-        if (not in_rollback) and (not variant):
+        if (not in_rollback) and (not variant) and (not multiple):
             return d
 
         d = d.copy()
 
-        style = d["style"]
-
         if isinstance(style, basestring):
+
+            if multiple is not None:
+                style = "block{}_multiple{}_{}".format(multiple[0], multiple[1], style)
+
             style = getattr(renpy.store.style, style)
 
             if variant is not None:
@@ -170,11 +179,15 @@ def compute_widget_properties(who_args, what_args, window_args, properties, vari
 
         return d
 
-    who_args = style_args(who_args)
-    what_args = style_args(what_args)
-    window_args = style_args(window_args)
+    who_args = style_args(who_args, "who")
+    what_args = style_args(what_args, "what")
+    window_args = style_args(window_args, "window")
 
     rv = dict(properties)
+
+    for prefix in renpy.config.character_id_prefixes:
+        rv[prefix] = style_args(properties.get(prefix, {}), prefix)
+
     rv["window"] = window_args
     rv["what"] = what_args
     rv["who"] = who_args
@@ -192,6 +205,7 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
                      screen=None,
                      layer=None,
                      properties={},
+                     multiple=None,
                      **kwargs):
     """
     This is called (by default) by renpy.display_say to add the
@@ -226,7 +240,7 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
     displaying the what text.
     """
 
-    props = compute_widget_properties(who_args, what_args, window_args, properties, variant=variant)
+    props = compute_widget_properties(who_args, what_args, window_args, properties, variant=variant, multiple=multiple)
 
     def handle_who():
         if who:
@@ -258,9 +272,13 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
         tag = screen
         index = 0
 
-        while renpy.exports.showing(tag):
-            index += 1
-            tag = "%s%d" % (screen, index)
+        if multiple:
+
+            if renpy.display.screen.has_screen("multiple_" + screen):
+                screen = "multiple_" + screen
+                kwargs["multiple"] = multiple
+
+            tag = "block{}_multiple{}_{}".format(multiple[0], multiple[1], tag)
 
         if image:
             kwargs["image"] = image
@@ -283,7 +301,7 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
 
         renpy.exports.shown_window()
 
-        return renpy.display.screen.get_widget(screen, "what", layer)
+        return (tag, "what", layer)
 
     # Apply the transform.
     if transform:
@@ -323,8 +341,9 @@ def show_display_say(who, what, who_args={}, what_args={}, window_args={},
 
 class SlowDone(object):
     delay = None
+    ctc_kwargs = { }
 
-    def __init__(self, ctc, ctc_position, callback, interact, type, cb_args, delay):  # @ReservedAssignment
+    def __init__(self, ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs):  # @ReservedAssignment
         self.ctc = ctc
         self.ctc_position = ctc_position
         self.callback = callback
@@ -332,22 +351,25 @@ class SlowDone(object):
         self.type = type
         self.cb_args = cb_args
         self.delay = delay
+        self.ctc_kwargs = ctc_kwargs
 
     def __call__(self):
 
-        if renpy.display.screen.has_screen("ctc"):
+        if self.interact and self.delay != 0:
 
-            if self.ctc:
-                args = [ self.ctc ]
-            else:
-                args = [ ]
+            if renpy.display.screen.has_screen("ctc"):
 
-            renpy.display.screen.show_screen("ctc", *args, _transient=True)
-            renpy.exports.restart_interaction()
+                if self.ctc:
+                    args = [ self.ctc ]
+                else:
+                    args = [ ]
 
-        elif self.ctc and self.ctc_position == "fixed":
-            renpy.display.screen.show_screen("_ctc", _transient=True, ctc=self.ctc)
-            renpy.exports.restart_interaction()
+                renpy.display.screen.show_screen("ctc", *args, _transient=True, _ignore_extra_kwargs=True, **self.ctc_kwargs)
+                renpy.exports.restart_interaction()
+
+            elif self.ctc and self.ctc_position == "fixed":
+                renpy.display.screen.show_screen("_ctc", _transient=True, ctc=self.ctc)
+                renpy.exports.restart_interaction()
 
         if self.delay is not None:
             renpy.ui.pausebehavior(self.delay, True, voice=True)
@@ -359,6 +381,8 @@ class SlowDone(object):
 
 # This function takes care of repeatably showing the screen as part of
 # an interaction.
+
+
 def display_say(
         who,
         what,
@@ -377,12 +401,25 @@ def display_say(
         checkpoint=True,
         ctc_timedpause=None,
         ctc_force=False,
-        advance=True):
+        advance=True,
+        multiple=None):
 
-    if not interact:
+    # Final is true if this statement should perform an interaction.
+
+    if multiple is None:
+        final = interact
+    else:
+        step, total = multiple
+
+        if step == total:
+            final = interact
+        else:
+            final = False
+
+    if not final:
         advance = False
 
-    if interact and (not renpy.game.preferences.skip_unseen) and (not renpy.game.context().seen_current(True)) and renpy.config.skipping == "fast":
+    if final and (not renpy.game.preferences.skip_unseen) and (not renpy.game.context().seen_current(True)) and renpy.config.skipping == "fast":
         renpy.config.skipping = None
 
     # If we're in fast skipping mode, don't bother with say
@@ -468,11 +505,21 @@ def display_say(
             # Figure out the CTC to use, if any.
             if last_pause:
                 what_ctc = ctc
+                ctc_kind = "last"
             else:
                 if delay is not None:
                     what_ctc = ctc_timedpause or ctc_pause
+                    ctc_kind = "timedpause"
                 else:
                     what_ctc = ctc_pause
+                    ctc_kind = "pause"
+
+            ctc_kwargs = {
+                "ctc_kind" : ctc_kind,
+                "ctc_last" : ctc,
+                "ctc_pause" : ctc_pause,
+                "ctc_timedpause" : ctc_timedpause,
+            }
 
             if not (interact or ctc_force):
                 what_ctc = None
@@ -486,40 +533,61 @@ def display_say(
             if delay == 0:
                 what_ctc = None
 
-            # Create the callback that is called when the slow text is done.
-            slow_done = SlowDone(what_ctc, ctc_position, callback, interact, type, cb_args, delay)
-
             # Run the show callback.
             for c in callback:
                 c("show", interact=interact, type=type, **cb_args)
 
+            # Create the callback that is called when the slow text is done.
+            slow_done = SlowDone(what_ctc, ctc_position, callback, interact, type, cb_args, delay, ctc_kwargs)
+
             # Show the text.
-            what_text = show_function(who, what_string)
+            if multiple:
+                what_text = show_function(who, what_string, multiple=multiple)
+            else:
+                what_text = show_function(who, what_string)
 
-            if not isinstance(what_text, renpy.text.text.Text):  # @UndefinedVariable
-                raise Exception("The say screen (or show_function) must return a Text object.")
+            if interact or what_string or (what_ctc is not None) or (behavior and afm):
 
-            if what_ctc and ctc_position == "nestled":
-                what_text.set_ctc(what_ctc)
+                if isinstance(what_text, tuple):
+                    what_text = renpy.display.screen.get_widget(what_text[0], what_text[1], what_text[2])
 
-            # Update the properties of the what_text widget.
-            what_text.start = start
-            what_text.end = end
-            what_text.slow = slow
-            what_text.slow_done = slow_done
+                if not isinstance(what_text, renpy.text.text.Text):  # @UndefinedVariable
+                    raise Exception("The say screen (or show_function) must return a Text object.")
 
-            what_text.update()
+                if what_ctc:
+
+                    if ctc_position == "nestled":
+                        what_text.set_ctc(what_ctc)
+                    elif ctc_position == "nestled-close":
+                        what_text.set_ctc([ u"\ufeff", what_ctc])
+
+                if what_text.text[0] == what_string:
+
+                    # Update the properties of the what_text widget.
+                    what_text.start = start
+                    what_text.end = end
+                    what_text.slow = slow
+                    what_text.slow_done = slow_done
+
+                    what_text.update()
+
+                elif renpy.config.developer:
+                    raise Exception("The displayable with id 'what' was not given the exact contents of the what variable given to the say screen.")
+
+                if behavior and afm:
+                    behavior.set_text(what_text)
+
+            else:
+
+                slow = False
 
             for c in callback:
                 c("show_done", interact=interact, type=type, **cb_args)
 
-            if behavior and afm:
-                behavior.set_text(what_text)
-
             if not slow:
                 slow_done()
 
-            if interact:
+            if final:
                 rv = renpy.ui.interact(mouse='say', type=type, roll_forward=roll_forward)
 
                 # This is only the case if the user has rolled forward, {nw} happens, or
@@ -539,7 +607,7 @@ def display_say(
         exception = e
 
     # Do the checkpoint and with None.
-    if interact:
+    if final:
 
         if not dtt.no_wait:
             if checkpoint:
@@ -554,9 +622,13 @@ def display_say(
         if with_none is None:
             with_none = renpy.config.implicit_with_none
 
+        renpy.plog(1, "before with none")
+
         if with_none:
             renpy.game.interface.do_with(None, None)
 
+        renpy.plog(1, "after with none")
+
     for c in callback:
         c("end", interact=interact, type=type, **cb_args)
 
@@ -572,6 +644,8 @@ class HistoryEntry(renpy.object.Object):
 
     # See ADVCharacter.add_history for the fields.
 
+    multiple = None
+
     def __repr__(self):
         return "<History {!r} {!r}>".format(self.who, self.what)
 
@@ -580,6 +654,11 @@ class HistoryEntry(renpy.object.Object):
 NotSet = renpy.object.Sentinel("NotSet")
 
 
+# The number of multiple characters we've seen during the current
+# interaction.
+multiple_count = 0
+
+
 class ADVCharacter(object):
     """
     The character object contains information about a character. When
@@ -715,24 +794,40 @@ class ADVCharacter(object):
         return type(self)(name, kind=self, **properties)
 
     # This is called before the interaction.
-    def do_add(self, who, what):
+    def do_add(self, who, what, multiple=None):
         return
 
     # This is what shows the screen for a given interaction.
-    def do_show(self, who, what):
-        return self.show_function(
-            who,
-            what,
-            who_args=self.who_args,
-            what_args=self.what_args,
-            window_args=self.window_args,
-            screen=self.screen,
-            properties=self.properties,
-            **self.show_args)
+    def do_show(self, who, what, multiple=None):
+
+        if multiple is not None:
+
+            return self.show_function(
+                who,
+                what,
+                who_args=self.who_args,
+                what_args=self.what_args,
+                window_args=self.window_args,
+                screen=self.screen,
+                properties=self.properties,
+                multiple=multiple,
+                **self.show_args)
+
+        else:
+
+            return self.show_function(
+                who,
+                what,
+                who_args=self.who_args,
+                what_args=self.what_args,
+                window_args=self.window_args,
+                screen=self.screen,
+                properties=self.properties,
+                **self.show_args)
 
     # This is called after the last interaction is done.
-    def do_done(self, who, what):
-        self.add_history("adv", who, what)
+    def do_done(self, who, what, multiple=None):
+        self.add_history("adv", who, what, multiple=multiple)
 
     # This is called when an extend occurs, before the usual add/show
     # cycel.
@@ -759,14 +854,12 @@ class ADVCharacter(object):
             properties=self.properties,
             **self.show_args)
 
-    def resolve_say_attributes(self, predict, wanted=[], remove=[]):
+    def resolve_say_attributes(self, mode, predict, attrs, wanted=[], remove=[], skip_trans=False):
         """
         Deals with image attributes associated with the current say
         statement.
         """
 
-        attrs = renpy.exports.get_say_attributes()
-
         if not (attrs or wanted or remove):
             return
 
@@ -778,8 +871,10 @@ class ADVCharacter(object):
 
         if attrs is None:
             attrs = ()
+        else:
+            attrs = tuple(attrs)
 
-        tagged_attrs = (self.image_tag,) + attrs
+        tagged_attrs = ( self.image_tag,) + attrs
         images = renpy.game.context().images
 
         layer = renpy.config.tag_layer.get(self.image_tag, "master")
@@ -795,23 +890,125 @@ class ADVCharacter(object):
             if images.showing(layer, new_image, exact=True):
                 return
 
+            show_image = (self.image_tag,) + attrs + tuple(wanted) + tuple( "-" + i for i in remove)
+
             if predict:
-                images.predict_show(new_image)
+                images.predict_show(show_image)
+
             else:
-                trans = renpy.config.say_attribute_transition
+                trans, layer = renpy.config.say_attribute_transition_callback(self.image_tag, mode)
 
-                if trans is not None:
-                    renpy.exports.with_statement(None)
+                if not skip_trans:
+                    if (trans is not None) and (layer is not None):
+                        renpy.exports.with_statement(None)
 
-                renpy.exports.show(new_image)
+                renpy.exports.show(show_image)
 
-                if trans is not None:
-                    renpy.exports.with_statement(trans)
+                if not skip_trans:
+                    if trans is not None:
+                        if layer is None:
+                            renpy.exports.with_statement(trans)
+                        else:
+                            renpy.exports.transition(trans, layer=layer)
 
         else:
 
-            # Otherwise, just record the attributes of the image.
-            images.predict_show("master", tagged_attrs, show=False)
+            if renpy.config.say_attributes_use_side_image:
+
+                tagged_attrs = (renpy.config.side_image_prefix_tag,) + tagged_attrs
+
+                new_image = images.apply_attributes(layer, self.image_tag, tagged_attrs, wanted, remove)
+
+                if new_image is None:
+                    new_image = tagged_attrs
+
+                images.predict_show(layer, new_image[1:], show=False)
+
+            else:
+
+                # Otherwise, just record the attributes of the image.
+                images.predict_show(layer, tagged_attrs, show=False)
+
+    def handle_say_attributes(self, predicting, interact):
+
+        attrs = renpy.game.context().say_attributes
+        renpy.game.context().say_attributes = None
+
+        temporary_attrs = renpy.game.context().temporary_attributes
+        renpy.game.context().say_attributes = None
+
+        if interact:
+            if temporary_attrs:
+                temporary_attrs = list(temporary_attrs)
+            else:
+                temporary_attrs = [ ]
+
+            # Prepend speaking_attribute, if present. This allows it to
+            # be suppressed by a negative temporary_attr, if desired.
+            if renpy.config.speaking_attribute is not None:
+                temporary_attrs.insert(0, renpy.config.speaking_attribute)
+
+        self.resolve_say_attributes("permanent", predicting, attrs, skip_trans=temporary_attrs)
+
+        # This is so late to give resolve_say_attributes time to do some
+        # error handling.
+        if not self.image_tag:
+            return None
+
+        if not temporary_attrs:
+            return None
+
+        images = renpy.game.context().images
+        attrs = images.get_attributes(None, self.image_tag)
+
+        self.resolve_say_attributes("temporary", predicting, temporary_attrs)
+
+        return (attrs, images)
+
+    def restore_say_attributes(self, predicting, state, interact):
+
+        if state is None:
+            return
+
+        attrs, images = state
+
+        if not self.image_tag:
+            return
+
+        # This is False when the context changes.
+        if images is not renpy.game.context().images:
+            return
+
+        current_attrs = images.get_attributes(None, self.image_tag)
+
+        if attrs == current_attrs:
+            return
+
+        image_with_attrs = (self.image_tag,) + attrs + tuple("-" + i for i in current_attrs if i not in attrs)
+
+        if images.showing(None, (self.image_tag,)):
+
+            if not predicting:
+                trans, layer = renpy.config.say_attribute_transition_callback(self.image_tag, "restore")
+
+                if interact:
+                    if (trans is not None) and (layer is not None):
+                        renpy.exports.with_statement(None)
+
+                renpy.exports.show(image_with_attrs)
+
+                if interact:
+                    if trans is not None:
+                        if layer is None:
+                            renpy.exports.with_statement(trans)
+                        else:
+                            renpy.exports.transition(trans, layer=layer)
+
+            else:
+                images.predict_show(None, image_with_attrs)
+
+        else:
+            images.predict_show(None, image_with_attrs, show=False)
 
     def __unicode__(self):
 
@@ -832,12 +1029,16 @@ class ADVCharacter(object):
         return "<Character: {!r}>".format(self.name)
 
     def empty_window(self):
+        if renpy.config.fast_empty_window and (self.name is None) and not (self.what_prefix or self.what_suffix):
+            self.do_show(None, "")
+            return
+
         self("", interact=False, _call_done=False)
 
-    def __call__(self, what, interact=True, _call_done=True, **kwargs):
+    def __call__(self, what, interact=True, _call_done=True, multiple=None, **kwargs):
 
         if kwargs:
-            return Character(kind=self, **kwargs)(what, interact=interact, _call_done=_call_done)
+            return Character(kind=self, **kwargs)(what, interact=interact, _call_done=_call_done, multiple=multiple)
 
         # Check self.condition to see if we should show this line at all.
         if not (self.condition is None or renpy.python.py_eval(self.condition)):
@@ -846,21 +1047,36 @@ class ADVCharacter(object):
         if not isinstance(what, basestring):
             raise Exception("Character expects its what argument to be a string, got %r." % (what,))
 
-        if interact and (renpy.config.speaking_attribute is not None):
-            speaking = [ renpy.config.speaking_attribute ]
+        # Figure out multiple and final. Multiple is None if this is not a multiple
+        # dialogue, or a step and the total number of steps in a multiple interaction.
+
+        global multiple_count
+
+        if multiple is None:
+            multiple_count = 0
+
         else:
-            speaking = [ ]
+            multiple_count += 1
+            multiple = (multiple_count, multiple)
 
-        self.resolve_say_attributes(False, wanted=speaking)
+            if multiple_count == multiple[1]:
+                multiple_count = 0
 
-        old_side_image_attributes = renpy.store._side_image_attributes
+        if multiple is None:
 
-        if self.image_tag:
-            attrs = (self.image_tag,) + renpy.game.context().images.get_attributes("master", self.image_tag)
-        else:
-            attrs = None
+            old_attr_state = self.handle_say_attributes(False, interact)
 
-        renpy.store._side_image_attributes = attrs
+            old_side_image_attributes = renpy.store._side_image_attributes
+
+            if self.image_tag:
+                attrs = (self.image_tag,) + renpy.game.context().images.get_attributes(None, self.image_tag)
+            else:
+                attrs = None
+
+            renpy.store._side_image_attributes = attrs
+
+            if not interact:
+                renpy.store._side_image_attributes_reset = True
 
         if renpy.config.voice_tag_callback is not None:
             renpy.config.voice_tag_callback(self.voice_tag)
@@ -874,6 +1090,9 @@ class ADVCharacter(object):
             display_args = self.display_args.copy()
             display_args["interact"] = display_args["interact"] and interact
 
+            if multiple is not None:
+                display_args["multiple"] = multiple
+
             who = self.name
 
             # If dynamic is set, evaluate the name expression.
@@ -890,46 +1109,47 @@ class ADVCharacter(object):
                 else:
                     who = self.who_prefix + who + self.who_suffix
 
-            ctx = renpy.game.context()
-
-            if ctx.translate_block_language is not None:
-                translate = False
-            else:
-                translate = True
-
             if renpy.config.new_substitutions:
                 what_pattern = sub(self.what_prefix + "[[what]" + self.what_suffix)
-                what = what_pattern.replace("[what]", sub(what, translate=translate))
+                what = what_pattern.replace("[what]", sub(what, translate=True))
             else:
                 what = self.what_prefix + what + self.what_suffix
 
             # Run the add_function, to add this character to the
             # things like NVL-mode.
-            self.do_add(who, what)
+
+            if multiple is not None:
+                self.do_add(who, what, multiple=multiple)
+            else:
+                self.do_add(who, what)
 
             # Now, display the damned thing.
             self.do_display(who, what, cb_args=self.cb_args, **display_args)
 
             # Indicate that we're done.
             if _call_done:
-                self.do_done(who, what)
+
+                if multiple is not None:
+                    self.do_done(who, what, multiple=multiple)
+                else:
+                    self.do_done(who, what)
 
                 # Finally, log this line of dialogue.
                 if who and isinstance(who, (str, unicode)):
                     renpy.exports.log(who)
+
                 renpy.exports.log(what)
                 renpy.exports.log("")
 
         finally:
 
-            if interact:
+            if (multiple is None) and interact:
                 renpy.store._side_image_attributes = old_side_image_attributes
-
-                self.resolve_say_attributes(False, remove=speaking)
+                self.restore_say_attributes(False, old_attr_state, interact)
 
     def predict(self, what):
 
-        self.resolve_say_attributes(True)
+        old_attr_state = self.handle_say_attributes(True, True)
 
         if renpy.config.speaking_attribute is not None:
             self.resolve_say_attributes(True, wanted=[ renpy.config.speaking_attribute ])
@@ -954,6 +1174,7 @@ class ADVCharacter(object):
 
         finally:
             renpy.store._side_image_attributes = old_side_image_attributes
+            self.restore_say_attributes(True, old_attr_state, True)
 
     def will_interact(self):
 
@@ -962,7 +1183,7 @@ class ADVCharacter(object):
 
         return self.display_args['interact']
 
-    def add_history(self, kind, who, what, **kwargs):
+    def add_history(self, kind, who, what, multiple=None, **kwargs):
         """
         This is intended to be called by subclasses of ADVCharacter to add
         History entries to _history_list.
@@ -992,6 +1213,8 @@ class ADVCharacter(object):
 
         h.image_tag = self.image_tag
 
+        h.multiple = multiple
+
         if renpy.game.context().rollback:
             h.rollback_identifier = renpy.game.log.current.identifier
         else:
@@ -1036,17 +1259,17 @@ def Character(name=NotSet, kind=None, **properties):
 
     `name`
         If a string, the name of the character for dialogue. When
-        `name` is ``None``, display of the name is omitted, as for
+        ``name`` is None, display of the name is omitted, as for
         narration.
 
     `kind`
         The Character to base this Character off of. When used, the
         default value of any argument not supplied to this Character
-        is the value of that argument supplied to `kind`. This can
+        is the value of that argument supplied to ``kind``. This can
         be used to define a template character, and then copy that
         character with changes.
 
-    **Linked Image**
+    **Linked Image.**
     An image tag may be associated with a Character. This allows a
     say statement involving this character to display an image with
     the tag, and also allows Ren'Py to automatically select a side
@@ -1056,7 +1279,7 @@ def Character(name=NotSet, kind=None, **properties):
          A string giving the image tag that is linked with this
          character.
 
-    **Voice Tag**
+    **Voice Tag.**
     If a voice tag is assign to a Character, the voice files that are
     associated with it, can be muted or played in the preference
     screen.
@@ -1090,7 +1313,7 @@ def Character(name=NotSet, kind=None, **properties):
     These options help to control the display of the name.
 
     `dynamic`
-        If true, then `name` should be a string containing a python
+        If true, then `name` should be a string containing a Python
         expression. That string will be evaluated before each line
         of dialogue, and the result used as the name of the character.
 
@@ -1099,7 +1322,7 @@ def Character(name=NotSet, kind=None, **properties):
     interaction occurs, and the mode that is entered upon display.
 
     `condition`
-        If given, this should be a string containing a python
+        If given, this should be a string containing a Python
         expression. If the expression is false, the dialogue
         does not occur, as if the say statement did not happen.
 
@@ -1109,9 +1332,9 @@ def Character(name=NotSet, kind=None, **properties):
         and additional elements can be added to the screen.
 
     `advance`
-        If True, the default, the player can click to advance through
+        If true, the default, the player can click to advance through
         the statement, and other means of advancing (such as skip and
-        auto-forward mode) will also work. If False, the player will be
+        auto-forward mode) will also work. If false, the player will be
         unable to move past the say statement unless an alternate means
         (such as a jump hyperlink or screen) is provided.
 
@@ -1129,23 +1352,25 @@ def Character(name=NotSet, kind=None, **properties):
     finished displaying, to prompt the user to advance.
 
     `ctc`
-        A Displayable to use as the click-to-continue indicator, unless
+        A displayable to use as the click-to-continue indicator, unless
         a more specific indicator is used.
 
     `ctc_pause`
-        A Displayable to use a the click-to-continue indicator when the
+        A displayable to use a the click-to-continue indicator when the
         display of text is paused by the {p} or {w} text tags.
 
     `ctc_timedpause`
-        A Displayable to use a the click-to-continue indicator when the
+        A displayable to use a the click-to-continue indicator when the
         display of text is paused by the {p=} or {w=} text tags. When
-        None, this takes its default from ctc_pause, use ``Null()``
-        when you want a ctc_pause but no ctc_timedpause.
+        None, this takes its default from `ctc_pause`, use ``Null()``
+        when you want a `ctc_pause` but no `ctc_timedpause`.
 
     `ctc_position`
         Controls the location of the click-to-continue indicator. If
         ``"nestled"``, the indicator is displayed as part of the text
-        being shown, immediately after the last character. If ``"fixed"``,
+        being shown, immediately after the last character. ``"nestled-close"`` is
+        similar, except a break is not allowed between the text and the CTC
+        indicator. If ``"fixed"``,
         the indicator is added to the screen, and its position is
         controlled by the position style properties.
 
@@ -1160,18 +1385,18 @@ def Character(name=NotSet, kind=None, **properties):
     Keyword arguments beginning with ``show_`` have the prefix
     stripped off, and are passed to the screen as arguments. For
     example, the value of ``show_myflag`` will become the value of
-    the ``myflag`` variable in the screen. (The myflag variable isn't
+    the ``myflag`` variable in the screen. (The ``myflag`` variable isn't
     used by default, but can be used by a custom say screen.)
 
     One show variable is, for historical reasons, handled by Ren'Py itself:
 
     `show_layer`
         If given, this should be a string giving the name of the layer
-        to show the "say" screen on.
+        to show the say screen on.
 
     **Styling Text and Windows.**
     Keyword arguments beginning with ``who_``, ``what_``, and
-    `window_`` have their prefix stripped, and are used to :ref:`style
+    ``window_`` have their prefix stripped, and are used to :ref:`style
     <styles>` the character name, the spoken text, and the window
     containing both, respectively.
 
@@ -1182,8 +1407,8 @@ def Character(name=NotSet, kind=None, **properties):
     dialogue.
 
     The style applied to the character name, spoken text, and window
-    can also be set this way, using the `who_style`, `what_style`, and
-    `window_style` arguments, respectively.
+    can also be set this way, using the ``who_style``, ``what_style``, and
+    ``window_style`` arguments, respectively.
 
     Setting :var:`config.character_id_prefixes` makes it possible to style
     other displayables as well. For example, when the default GUI is used,
