@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,6 +22,8 @@
 # This file contains classes that handle layout of displayables on
 # the screen.
 
+from __future__ import print_function
+
 from renpy.display.render import render, Render
 import renpy.display
 
@@ -103,6 +105,13 @@ class Container(renpy.display.core.Displayable):
 
         super(Container, self).__init__(**properties)
 
+    def _handles_event(self, event):
+        for i in self.children:
+            if i._handles_event(event):
+                return True
+
+        return False
+
     def set_style_prefix(self, prefix, root):
         super(Container, self).set_style_prefix(prefix, root)
 
@@ -111,6 +120,9 @@ class Container(renpy.display.core.Displayable):
 
     def _duplicate(self, args):
 
+        if args and args.args:
+            args.extraneous()
+
         if not self._duplicatable:
             return self
 
@@ -252,8 +264,9 @@ class Container(renpy.display.core.Displayable):
         return False
 
 
-def LiveComposite(size, *args, **properties):
+def Composite(size, *args, **properties):
     """
+    :name: Composite
     :doc: disp_imagelike
 
     This creates a new displayable of `size`, by compositing other
@@ -269,7 +282,7 @@ def LiveComposite(size, *args, **properties):
 
     ::
 
-       image eileen composite = LiveComposite(
+       image eileen composite = Composite(
            (300, 600),
            (0, 0), "body.png",
            (0, 0), "clothes.png",
@@ -292,8 +305,13 @@ def LiveComposite(size, *args, **properties):
     return rv
 
 
+LiveComposite = Composite
+
+
 class Position(Container):
     """
+    :undocumented:
+
     Controls the placement of a displayable on the screen, using
     supplied position properties. This is the non-curried form of
     Position, which should be used when the user has directly created
@@ -397,8 +415,15 @@ class Grid(Container):
 
     def render(self, width, height, st, at):
 
+        xspacing = self.style.xspacing
+        yspacing = self.style.yspacing
+
+        if xspacing is None:
+            xspacing = self.style.spacing
+        if yspacing is None:
+            yspacing = self.style.spacing
+
         # For convenience and speed.
-        padding = self.style.spacing
         cols = self.cols
         rows = self.rows
 
@@ -423,9 +448,9 @@ class Grid(Container):
         renheight = height
 
         if self.style.xfill:
-            renwidth = (width - (cols - 1) * padding) / cols
+            renwidth = (width - (cols - 1) * xspacing) / cols
         if self.style.yfill:
-            renheight = (height - (rows - 1) * padding) / rows
+            renheight = (height - (rows - 1) * yspacing) / rows
 
         renders = [ render(i, renwidth, renheight, st, at) for i in children ]
         sizes = [ i.get_size() for i in renders ]
@@ -443,8 +468,8 @@ class Grid(Container):
         if self.style.yfill:
             cheight = renheight
 
-        width = cwidth * cols + padding * (cols - 1)
-        height = cheight * rows + padding * (rows - 1)
+        width = cwidth * cols + xspacing * (cols - 1)
+        height = cheight * rows + yspacing * (rows - 1)
 
         rv = renpy.display.render.Render(width, height)
 
@@ -456,8 +481,8 @@ class Grid(Container):
                 child = children[ x + y * cols ]
                 surf = renders[x + y * cols]
 
-                xpos = x * (cwidth + padding)
-                ypos = y * (cheight + padding)
+                xpos = x * (cwidth + xspacing)
+                ypos = y * (cheight + yspacing)
 
                 offset = child.place(rv, xpos, ypos, cwidth, cheight, surf)
                 offsets.append(offset)
@@ -487,6 +512,7 @@ class MultiBox(Container):
     layer_name = None
     first = True
     order_reverse = False
+    layout = None
 
     def __init__(self, spacing=None, layout=None, style='default', **properties):
 
@@ -495,6 +521,8 @@ class MultiBox(Container):
 
         super(MultiBox, self).__init__(style=style, **properties)
 
+        self._clipping = self.style.clipping
+
         self.default_layout = layout
 
         # The start and animation times for children of this
@@ -607,6 +635,16 @@ class MultiBox(Container):
 
         self.scene_list.extend(l)
 
+    def update_times(self):
+
+        it = renpy.game.interface.interact_time
+
+        self.start_times = [ i or it for i in self.start_times ]
+        self.anim_times = [ i or it for i in self.anim_times ]
+
+        if it is None:
+            self.first = True
+
     def render(self, width, height, st, at):
 
         # Do we need to adjust the child times due to our being a layer?
@@ -628,28 +666,19 @@ class MultiBox(Container):
             self.first = False
 
             if adjust_times:
+                self.update_times()
 
-                it = renpy.game.interface.interact_time
+        layout = self.style.box_layout
 
-                self.start_times = [ i or it for i in self.start_times ]
-                self.anim_times = [ i or it for i in self.anim_times ]
-
-            layout = self.style.box_layout
-
-            if layout is None:
-                layout = self.default_layout
-
-            self.layout = layout  # W0201
-
-        else:
-            layout = self.layout
+        if layout is None:
+            layout = self.default_layout
 
         # Handle time adjustment, store the results in csts and cats.
         if adjust_times:
             t = renpy.game.interface.frame_time
 
-            csts = [ t - start for start in self.start_times ]
-            cats = [ t - anim for anim in self.anim_times ]
+            csts = [ 0 if (start is None) else (t - start) for start in self.start_times ]
+            cats = [ 0 if (anim is None) else (t - anim) for anim in self.anim_times ]
 
         else:
             csts = [ st ] * len(self.children)
@@ -755,6 +784,7 @@ class MultiBox(Container):
         spacings = [ first_spacing ] + [ spacing ] * (len(self.children) - 1)
 
         box_wrap = self.style.box_wrap
+        box_wrap_spacing = self.style.box_wrap_spacing
         xfill = self.style.xfill
         yfill = self.style.yfill
         xminimum = self.style.xminimum
@@ -856,7 +886,7 @@ class MultiBox(Container):
                 if box_wrap and remwidth - sw - padding < 0 and line:
                     maxx, maxy = layout_line(line, target_width - x, 0)
 
-                    y += line_height
+                    y += line_height + box_wrap_spacing
                     x = 0
                     line_height = 0
                     remwidth = width
@@ -898,7 +928,7 @@ class MultiBox(Container):
                 if box_wrap and remheight - sh - padding < 0:
                     maxx, maxy = layout_line(line, 0, target_height - y)
 
-                    x += line_width
+                    x += line_width + box_wrap_spacing
                     y = 0
                     line_width = 0
                     remheight = height
@@ -943,6 +973,14 @@ class MultiBox(Container):
 
     def event(self, ev, x, y, st):
 
+        # Do we need to adjust the child times due to our being a layer?
+        if self.first:
+
+            self.first = False
+
+            if self.layer_name or (self.layers is not None):
+                self.update_times()
+
         children_offsets = zip(self.children, self.offsets, self.start_times)
 
         if not self.style.order_reverse:
@@ -963,6 +1001,10 @@ class MultiBox(Container):
 
         except IgnoreLayers:
             if self.layers:
+
+                if ev.type != renpy.display.core.TIMEEVENT:
+                    renpy.display.interface.post_time_event()
+
                 return None
             else:
                 raise
@@ -1022,7 +1064,9 @@ class Window(Container):
             self.add(child)
 
     def visit(self):
-        return [ self.style.background ] + self.children
+        rv = [ ]
+        self.style._visit_window(rv.append)
+        return rv + self.children
 
     def get_child(self):
         return self.style.child or self.child
@@ -1037,10 +1081,6 @@ class Window(Container):
 
             group.members.append(self)
 
-    def predict_one(self):
-        pd = renpy.display.predict.displayable
-        self.style._predict_window(pd)
-
     def render(self, width, height, st, at):
 
         # save some typing.
@@ -1049,6 +1089,9 @@ class Window(Container):
         xminimum = scale(style.xminimum, width)
         yminimum = scale(style.yminimum, height)
 
+        xmaximum = scale(style.xmaximum, width)
+        ymaximum = scale(style.ymaximum, height)
+
         size_group = self.style.size_group
         if size_group and size_group in size_groups:
             xminimum = max(xminimum, size_groups[size_group].width(width, height, st, at))
@@ -1092,11 +1135,11 @@ class Window(Container):
 
         if renpy.config.enforce_window_max_size:
 
-            if style.xmaximum is not None:
-                width = min(width, style.xmaximum)
+            if xmaximum is not None:
+                width = min(width, xmaximum)
 
-            if style.ymaximum is not None:
-                height = min(height, style.ymaximum)
+            if ymaximum is not None:
+                height = min(height, ymaximum)
 
         rv = renpy.display.render.Render(width, height)
 
@@ -1168,18 +1211,18 @@ class DynamicDisplayable(renpy.display.core.Displayable):
 
     ::
 
-        # If tooltip is not empty, shows it in a text. Otherwise,
-        # show Null. Checks every tenth of a second to see if the
-        # tooltip has been updated.
+        # Shows a countdown from 5 to 0, updating it every tenth of
+        # a second until the time expires.
         init python:
-             def show_tooltip(st, at):
-                 if tooltip:
-                     return tooltip, .1
-                 else:
-                     return Null(), .1
 
-        image tooltipper = DynamicDisplayable(show_tooltip)
+            def show_countdown(st, at):
+                if st > 5.0:
+                    return Text("0.0"), None
+                else:
+                    d = Text("{:.1f}".format(5.0 - st))
+                    return d, 0.1
 
+        image countdown = DynamicDisplayable(show_countdown)
     """
 
     nosave = [ 'child' ]
@@ -1230,8 +1273,13 @@ class DynamicDisplayable(renpy.display.core.Displayable):
             else:
                 child, _ = self.function(0, 0, *self.args, **self.kwargs)
 
-            if child is not None:
+            if isinstance(child, list):
+
+                for i in child:
+                    renpy.display.predict.displayable(i)
+            else:
                 renpy.display.predict.displayable(child)
+
         except:
             pass
 
@@ -1245,6 +1293,7 @@ class DynamicDisplayable(renpy.display.core.Displayable):
         if self.child:
             return self.child.event(ev, x, y, st)
 
+
 # A cache of compiled conditions used by ConditionSwitch.
 cond_cache = { }
 
@@ -1270,13 +1319,16 @@ def condition_switch_pick(switch):
     raise Exception("Switch could not choose a displayable.")
 
 
-def condition_switch_show(st, at, switch):
+def condition_switch_show(st, at, switch, predict_all=None):
     return condition_switch_pick(switch), None
 
 
-def condition_switch_predict(switch):
+def condition_switch_predict(switch, predict_all=None):
 
-    if renpy.game.lint:
+    if predict_all is None:
+        predict_all = renpy.config.conditionswitch_predict_all
+
+    if renpy.game.lint or (predict_all and renpy.display.predict.predicting):
         return [ d for _cond, d in switch ]
 
     return [ condition_switch_pick(switch) ]
@@ -1284,18 +1336,28 @@ def condition_switch_predict(switch):
 
 def ConditionSwitch(*args, **kwargs):
     """
+    :name: ConditionSwitch
     :doc: disp_dynamic
+    :args: (*args, predict_all=None, **properties)
 
     This is a displayable that changes what it is showing based on
-    python conditions. The positional argument should be given in
+    Python conditions. The positional arguments should be given in
     groups of two, where each group consists of:
 
-    * A string containing a python condition.
+    * A string containing a Python condition.
     * A displayable to use if the condition is true.
 
     The first true condition has its displayable shown, at least
     one condition should always be true.
 
+    The conditions uses here should not have externally-visible side-effects.
+
+    `predict_all`
+        If True, all of the possible displayables will be predicted when
+        the displayable is shown. If False, only the current condition is
+        predicted. If None, :var:`config.conditionswitch_predict_all` is
+        used.
+
     ::
 
         image jill = ConditionSwitch(
@@ -1303,6 +1365,7 @@ def ConditionSwitch(*args, **kwargs):
             "True", "jill_sober.png")
     """
 
+    predict_all = kwargs.pop("predict_all", None)
     kwargs.setdefault('style', 'default')
 
     switch = [ ]
@@ -1321,6 +1384,7 @@ def ConditionSwitch(*args, **kwargs):
 
     rv = DynamicDisplayable(condition_switch_show,
                             switch,
+                            predict_all,
                             _predict_function=condition_switch_predict)
 
     return Position(rv, **kwargs)
@@ -1329,6 +1393,7 @@ def ConditionSwitch(*args, **kwargs):
 def ShowingSwitch(*args, **kwargs):
     """
     :doc: disp_dynamic
+    :args: (*args, predict_all=None, **properties)
 
     This is a displayable that changes what it is showing based on the
     images are showing on the screen. The positional argument should
@@ -1339,6 +1404,12 @@ def ShowingSwitch(*args, **kwargs):
 
     A default image should be specified.
 
+    `predict_all`
+        If True, all of the possible displayables will be predicted when
+        the displayable is shown. If False, only the current condition is
+        predicted. If None, :var:`config.conditionswitch_predict_all` is
+        used.
+
     One use of ShowingSwitch is to have side images change depending on
     the current emotion of a character. For example::
 
@@ -1394,19 +1465,23 @@ class IgnoresEvents(Container):
         return None
 
 
-def LiveCrop(rect, child, **properties):
+def Crop(rect, child, **properties):
     """
     :doc: disp_imagelike
+    :name: Crop
 
-    This created a displayable by cropping `child` to `rect`, where
+    This creates a displayable by cropping `child` to `rect`, where
     `rect` is an (x, y, width, height) tuple. ::
 
-        image eileen cropped = LiveCrop((0, 0, 300, 300), "eileen happy")
+        image eileen cropped = Crop((0, 0, 300, 300), "eileen happy")
     """
 
     return renpy.display.motion.Transform(child, crop=rect, **properties)
 
 
+LiveCrop = Crop
+
+
 class Side(Container):
 
     possible_positions = set([ 'tl', 't', 'tr', 'r', 'br', 'b', 'bl', 'l', 'c'])
@@ -1445,6 +1520,9 @@ class Side(Container):
         super(Side, self)._clear()
         self.sized = False
 
+    def per_interact(self):
+        self.sized = False
+
     def render(self, width, height, st, at):
 
         pos_d = { }
@@ -1640,16 +1718,31 @@ class AdjustTimes(Container):
 
         self.add(child)
 
+    def adjusted_times(self):
+
+        interact_time = renpy.game.interface.interact_time
+
+        if (self.start_time is None) and (interact_time is not None):
+            self.start_time = interact_time
+
+        if self.start_time is not None:
+            st = renpy.game.interface.frame_time - self.start_time
+        else:
+            st = 0
+
+        if (self.anim_time is None) and (interact_time is not None):
+            self.anim_time = interact_time
+
+        if self.anim_time is not None:
+            at = renpy.game.interface.frame_time - self.anim_time
+        else:
+            at = 0
+
+        return st, at
+
     def render(self, w, h, st, at):
 
-        if self.start_time is None:
-            self.start_time = renpy.game.interface.frame_time
-
-        if self.anim_time is None:
-            self.anim_time = renpy.game.interface.frame_time
-
-        st = renpy.game.interface.frame_time - self.start_time
-        at = renpy.game.interface.frame_time - self.anim_time
+        st, at = self.adjusted_times()
 
         cr = renpy.display.render.render(self.child, w, h, st, at)
         cw, ch = cr.get_size()
@@ -1660,19 +1753,24 @@ class AdjustTimes(Container):
 
         return rv
 
+    def event(self, ev, x, y, st):
+        st, _ = self.adjusted_times()
+        Container.event(self, ev, x, y, st)
+
     def get_placement(self):
         return self.child.get_placement()
 
 
-class LiveTile(Container):
+class Tile(Container):
     """
     :doc: disp_imagelike
+    :name: Tile
 
     Tiles `child` until it fills the area allocated to this displayable.
 
     ::
 
-        image bg tile = LiveTile("bg.png")
+        image bg tile = Tile("bg.png")
 
     """
 
@@ -1708,6 +1806,9 @@ class LiveTile(Container):
         return rv
 
 
+LiveTile = Tile
+
+
 class Flatten(Container):
     """
     :doc: disp_imagelike
@@ -1758,11 +1859,7 @@ class AlphaMask(Container):
     opaque where `child` and `mask` are both opaque.
 
     The `child` and `mask` parameters may be arbitrary displayables. The
-    size of the AlphaMask is the size of the overlap between `child` and
-    `mask`.
-
-    Note that this takes different arguments from :func:`im.AlphaMask`,
-    which uses the mask's color channel.
+    size of the AlphaMask is the size of `child`.
     """
 
     def __init__(self, child, mask, **properties):
@@ -1771,24 +1868,19 @@ class AlphaMask(Container):
         self.add(child)
         self.mask = renpy.easy.displayable(mask)
         self.null = None
-        self.size = None
 
     def render(self, width, height, st, at):
 
         cr = renpy.display.render.render(self.child, width, height, st, at)
-        mr = renpy.display.render.render(self.mask, width, height, st, at)
+        w, h = cr.get_size()
 
-        cw, ch = cr.get_size()
-        mw, mh = mr.get_size()
+        mr = renpy.display.render.Render(w, h)
+        mr.place(self.mask, main=False)
 
-        w = min(cw, mw)
-        h = min(ch, mh)
-        size = (w, h)
+        if self.null is None:
+            self.null = Fixed()
 
-        if self.size != size:
-            self.null = Null(w, h)
-
-        nr = renpy.display.render.render(self.null, width, height, st, at)
+        nr = renpy.display.render.render(self.null, w, h, st, at)
 
         rv = renpy.display.render.Render(w, h, opaque=False)
 
