@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -28,6 +28,7 @@ import time
 
 import renpy.display
 import renpy.test
+from renpy import six
 
 pyast = __import__("ast", { })
 
@@ -51,10 +52,15 @@ def check_infinite_loop():
 
     global il_time
 
-    if time.time() > il_time:
-        il_time = time.time() + 60
+    now = time.time()
+
+    if now > il_time:
+        il_time = now + 60
         raise Exception("Possible infinite loop.")
 
+    if renpy.config.developer and (il_time > now + 60):
+        il_time = now + 60
+
     return
 
 
@@ -65,6 +71,11 @@ def not_infinite_loop(delay):
     Resets the infinite loop detection timer to `delay` seconds.
     """
 
+    # Give more time in non-developer mode, since computers can be crazy slow
+    # and the player can't do much about it.
+    if not renpy.config.developer:
+        delay *= 5
+
     global il_time
     il_time = time.time() + delay
 
@@ -96,6 +107,9 @@ class LineLogEntry(object):
 
         return (self.filename == other.filename) and (self.line == other.line) and (self.node is other.node)
 
+    def __ne__(self, other):
+        return not (self == other)
+
 
 class Context(renpy.object.Object):
     """
@@ -120,7 +134,7 @@ class Context(renpy.object.Object):
     does participates in rollback.
     """
 
-    __version__ = 13
+    __version__ = 16
 
     nosave = [ 'next_node' ]
 
@@ -128,6 +142,24 @@ class Context(renpy.object.Object):
 
     force_checkpoint = False
 
+    come_from_name = None
+    come_from_label = None
+
+    temporary_attributes = None
+
+    def __repr__(self):
+
+        if not self.current:
+            return "<Context>"
+
+        node = renpy.game.script.lookup(self.current)
+
+        return "<Context: {}:{} {!r}>".format(
+            node.filename,
+            node.linenumber,
+            node.diff_info(),
+            )
+
     def after_upgrade(self, version):
         if version < 1:
             self.scene_lists.image_predict_info = self.predict_info.images
@@ -166,12 +198,18 @@ class Context(renpy.object.Object):
         if version < 11:
             self.say_attributes = None
 
-        if version < 12:
-            self.translate_block_language = None
-
         if version < 13:
             self.line_log = [ ]
 
+        if version < 14:
+            self.movie = { }
+
+        if version < 15:
+            self.abnormal_stack = [ False ] * len(self.return_stack)
+
+        if version < 16:
+            self.alternate_translate_identifier = None
+
     def __init__(self, rollback, context=None, clear=False):
         """
         `clear`
@@ -184,6 +222,9 @@ class Context(renpy.object.Object):
         self.call_location_stack = [ ]
         self.return_stack = [ ]
 
+        # The value of abnormal at the time of the call.
+        self.abnormal_stack = [ ]
+
         # Two deeper then the return stack and call location stack.
         # 1 deeper is for the context top-level, 2 deeper is for
         # _args, _kwargs, and _return.
@@ -226,6 +267,7 @@ class Context(renpy.object.Object):
 
         # The attributes that are used by the current say statement.
         self.say_attributes = None
+        self.temporary_attributes = None
 
         # A list of lines that were run since the last time this log was
         # cleared.
@@ -235,15 +277,20 @@ class Context(renpy.object.Object):
         # executed?
         self.force_checkpoint = False
 
+        # A mapt from a channel to the Movie playing on that channel.
+        self.movie = { }
+
         if context:
             oldsl = context.scene_lists
             self.runtime = context.runtime
 
             vars(self.info).update(vars(context.info))
 
-            for k, v in context.music.iteritems():
+            for k, v in context.music.items():
                 self.music[k] = v.copy()
 
+            self.movie = dict(context.movie)
+
             self.images = renpy.display.image.ShownImageInfo(context.images)
 
         else:
@@ -269,8 +316,20 @@ class Context(renpy.object.Object):
         # The identifier of the current translate block.
         self.translate_identifier = None
 
-        # The language of the current translate block.
-        self.translate_block_language = None
+        # The alternate identifier of the current translate block.
+        self.alternate_translate_identifier = None
+
+    def replace_node(self, old, new):
+
+        def replace_one(name):
+            n = renpy.game.script.lookup(name)
+            if n is old:
+                return new.name
+
+            return name
+
+        self.current = replace_one(self.current)
+        self.return_stack = [ replace_one(i) for i in self.return_stack ]
 
     def make_dynamic(self, names, context=False):
         """
@@ -323,6 +382,18 @@ class Context(renpy.object.Object):
         while self.dynamic_stack:
             self.pop_dynamic()
 
+    def pop_dynamic_roots(self, roots):
+
+        for dynamic in reversed(self.dynamic_stack):
+
+            for k, v in dynamic.iteritems():
+                name = "store." + k
+
+                if isinstance(v, Delete) and (name in roots):
+                    del roots[name]
+                else:
+                    roots[name] = v
+
     def goto_label(self, node_name):
         """
         Sets the name of the node that will be run when this context
@@ -384,6 +455,14 @@ class Context(renpy.object.Object):
         code = compile(module, node.filename, 'exec')
         exec(code)
 
+    def come_from(self, name, label):
+        """
+        When control reaches name, call label. Only for internal use.
+        """
+
+        self.come_from_name = name
+        self.come_from_label = label
+
     def run(self, node=None):
         """
         Executes as many nodes as possible in the current context. If the
@@ -401,8 +480,22 @@ class Context(renpy.object.Object):
         developer = renpy.config.developer
         tracing = sys.gettrace() is not None
 
+        # Is this the first time through the loop?
+        first = True
+
         while node:
 
+            if node.name == self.come_from_name:
+                self.come_from_name = None
+                node = self.call(self.come_from_label, return_site=node.name)
+                self.make_dynamic([ "_return", "_begin_rollback" ])
+                renpy.store._begin_rollback = False
+
+            this_node = node
+            type_node_name = type(node).__name__
+
+            renpy.plog(1, "--- start {} ({}:{})", type_node_name, node.filename, node.linenumber)
+
             self.current = node.name
             self.last_abnormal = self.abnormal
             self.abnormal = False
@@ -414,12 +507,34 @@ class Context(renpy.object.Object):
                 if ll_entry not in self.line_log:
                     self.line_log.append(ll_entry)
 
-            if self.rollback and renpy.game.log:
-                renpy.game.log.begin()
+            if not renpy.store._begin_rollback:
+                update_rollback = False
+                force_rollback = False
+            elif first or self.force_checkpoint or (node.rollback == "force"):
+                update_rollback = True
+                force_rollback = True
+            elif not renpy.config.all_nodes_rollback and (node.rollback == "never"):
+                update_rollback = False
+                force_rollback = False
+            else:
+                update_rollback = True
+                force_rollback = False
 
-            if self.force_checkpoint:
-                renpy.game.log.checkpoint(hard=False)
-                self.force_checkpoint = False
+            # Force a new rollback to start to match things in the forward log.
+            if renpy.game.log.forward and renpy.game.log.forward[0][0] == node.name:
+                update_rollback = True
+                force_rollback = True
+
+            first = False
+
+            if update_rollback:
+
+                if self.rollback and renpy.game.log:
+                    renpy.game.log.begin(force=force_rollback)
+
+                if self.rollback and self.force_checkpoint:
+                    renpy.game.log.checkpoint(hard=False)
+                    self.force_checkpoint = False
 
             self.seen = False
 
@@ -435,8 +550,13 @@ class Context(renpy.object.Object):
                     renpy.game.exception_info = "While running game code:"
 
                     self.next_node = None
+
+                    renpy.plog(2, "    before execute {} ({}:{})", type_node_name, node.filename, node.linenumber)
+
                     node.execute()
 
+                    renpy.plog(2, "    after execute {} ({}:{})", type_node_name, node.filename, node.linenumber)
+
                     if developer and self.next_node:
                         self.check_stacks()
 
@@ -461,7 +581,7 @@ class Context(renpy.object.Object):
                     except renpy.game.CONTROL_EXCEPTIONS as ce:
                         raise ce
                     except Exception as ce:
-                        raise exc_info[0], exc_info[1], exc_info[2]
+                        six.reraise(exc_info[0], exc_info[1], exc_info[2])
 
                 node = self.next_node
 
@@ -471,10 +591,14 @@ class Context(renpy.object.Object):
 
             except renpy.game.CallException as e:
 
-                if self.next_node is None:
-                    raise Exception("renpy.call can't be used when the next node is undefined.")
+                if e.from_current:
+                    return_site = node.name
+                else:
+                    if self.next_node is None:
+                        raise Exception("renpy.call can't be used when the next node is undefined.")
+                    return_site = self.next_node.name
 
-                node = self.call(e.label, return_site=self.next_node.name)
+                node = self.call(e.label, return_site=return_site)
                 self.abnormal = True
                 renpy.store._args = e.args
                 renpy.store._kwargs = e.kwargs
@@ -483,8 +607,10 @@ class Context(renpy.object.Object):
                 renpy.game.persistent._seen_ever[self.current] = True  # @UndefinedVariable
                 renpy.game.seen_session[self.current] = True
 
-            if self.rollback and renpy.game.log:
-                renpy.game.log.complete()
+            renpy.plog(2, "    end {} ({}:{})", type_node_name, this_node.filename, this_node.linenumber)
+
+        if self.rollback and renpy.game.log:
+            renpy.game.log.complete()
 
     def mark_seen(self):
         """
@@ -508,6 +634,7 @@ class Context(renpy.object.Object):
 
         self.return_stack.append(return_site)
         self.dynamic_stack.append({ })
+        self.abnormal_stack.append(self.last_abnormal)
         self.current = label
 
         self.make_dynamic([ "_args", "_kwargs" ])
@@ -530,6 +657,7 @@ class Context(renpy.object.Object):
         self.return_stack.pop()
         self.call_location_stack.pop()
         self.pop_dynamic()
+        self.abnormal_stack.pop()
 
     def lookup_return(self, pop=True):
         """
@@ -554,12 +682,14 @@ class Context(renpy.object.Object):
                 self.return_stack.pop()
                 self.call_location_stack.pop()
                 self.pop_dynamic()
+                self.abnormal = self.abnormal_stack.pop()
 
                 continue
 
             if pop:
                 self.return_stack.pop()
                 self.call_location_stack.pop()
+                self.abnormal = self.abnormal_stack.pop()
 
             return node
 
@@ -582,6 +712,10 @@ class Context(renpy.object.Object):
         rv.translate_language = self.translate_language
         rv.translate_identifier = self.translate_identifier
 
+        rv.abnormal = self.abnormal
+        rv.last_abnormal = self.last_abnormal
+        rv.abnormal_stack = list(self.abnormal_stack)
+
         return rv
 
     def predict_call(self, label, return_site):
@@ -746,7 +880,8 @@ def run_context(top):
     the RestartContext and RestartTopContext exceptions.
     """
 
-    label = None
+    if renpy.config.context_callback is not None:
+        renpy.config.context_callback()
 
     while True:
 
@@ -754,10 +889,6 @@ def run_context(top):
 
             context = renpy.game.context()
 
-            if label and renpy.game.script.has_label(label):
-                context.call(label)
-            label = None
-
             context.run()
 
             rv = renpy.store._return
@@ -770,10 +901,6 @@ def run_context(top):
 
             # Apply defaults.
             renpy.exports.execute_default_statement(False)
-
-            if e.label:
-                label = e.label
-
             continue
 
         except renpy.game.RestartTopContext as e:
@@ -781,10 +908,6 @@ def run_context(top):
 
                 # Apply defaults.
                 renpy.exports.execute_default_statement(False)
-
-                if e.label:
-                    label = e.label
-
                 continue
 
             else:
