@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,6 +22,8 @@
 # This file contains support for string translation and string formatting
 # operations.
 
+from __future__ import print_function
+
 import renpy
 import string
 import os
@@ -167,16 +169,23 @@ class Formatter(string.Formatter):
         if "r" in conversion:
             value = repr(value)
         elif "s" in conversion:
-            value = str(value)
+            value = unicode(value)
 
         if "t" in conversion:
+            if not isinstance(value, basestring):
+                value = unicode(value)
+
             value = renpy.translation.translate_string(value)
 
         if "q" in conversion:
+            if not isinstance(value, basestring):
+                value = unicode(value)
+
             value = value.replace("{", "{{")
 
         return value
 
+
 # The instance of Formatter we use.
 formatter = Formatter()
 
@@ -212,6 +221,9 @@ def substitute(s, scope=None, force=False, translate=True):
     occurred, or False if no substitution occurred.
     """
 
+    if not isinstance(s, basestring):
+        s = unicode(s)
+
     if translate:
         s = renpy.translation.translate_string(s)
 
